/*********************************************************************
 *	  			OSCILLOPIC Firmware based on the
 *          Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        main.c
 * 
/** I N C L U D E S **********************************************************/
#include "p18f2550.h"
#include "typedefs.h"                  
#include "usb.h"                    
#include "interrupt.h"
#include "io_cfg.h"              
#include "user.h"         

/** V A R I A B L E S ********************************************************/

extern char ordre, tt1;

/** P R I V A T E  P R O T O T Y P E S ***************************************/
static void InitializeSystem(void);
void USBTasks(void);


/** D E C L A R A T I O N S **************************************************/
#pragma code InterruptVectorLow = 0x08
void high_isr (void) {
	_asm
		goto isr
	_endasm
}

#pragma code InterruptVectorHigh = 0x18
void low_isr (void) {
	_asm
		goto lowisr
	_endasm
}

#pragma code page
/******************************************************************************
 * Function:        void main(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Main program entry point.
 * Note:            None
 *****************************************************************************/
void main(void) {

  InitializeSystem();
  ordre=0;
  while(1) {
    USBTasks();         // USB Tasks
    ProcessIO();        // See user\user.c & .h
   }//end while
 }//end main

/******************************************************************************
 * Function:        static void InitializeSystem(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        InitializeSystem is a centralize initialization routine.
 *                  All required USB initialization routines are called from
 *                  here.
 *                  User application initialization routine should also be
 *                  called from here.                  
 *****************************************************************************/
static void InitializeSystem(void) {
    ADCON1 |= 0x0F;                 // Default all pins to digital
    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See io_cfg.h
    #endif
    
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;
    #endif
    
    UserInit();                     // See user.c & .h

    mInitializeUSBDriver();         // See usbdrv.h

}//end InitializeSystem

/******************************************************************************
 * Function:        void USBTasks(void)
 * PreCondition:    InitializeSystem has been called.
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Service loop for USB tasks.
 *****************************************************************************/
void USBTasks(void) {
    /* Servicing Hardware */
    USBCheckBusStatus();                    // Must use polling method
    if(UCFGbits.UTEYE!=1)
        USBDriverService();                 // Interrupt or polling method

}// end USBTasks

/** EOF main.c ***************************************************************/
