/**************************************************************************
 *G.Samblancat@free.fr						mars 2009
 *                       "Le Panneau aux 180 leds"
 *
 ***************************************************************************
 * FileName:        user.c
 * Processor:       PIC18
 ***************************************************************************/

/** I N C L U D E S ********************************************************/
#include "p18f2550.h"
#include "typedefs.h"
#include "usb.h"
#include "io_cfg.h"             // I/O pin mapping
#include "user.h"
#include "fonts.h"
#include "interrupt.h"
#include "reset.h"
#include "eeprom.h"
#include "timers.h"
#include "delays.h"
#include "portb.h"

/** C O N F I G U R A T I O N   D U   P I C  **********************************/
#pragma config USBDIV=2, CPUDIV=OSC3_PLL4, PLLDIV=3
#pragma config FOSC=HSPLL_HS, FCMEM=OFF, IESO=OFF
#pragma config VREGEN=ON, PWRT=ON, BOR=OFF
#pragma config WDT=OFF
#pragma config PBADEN=OFF, LPT1OSC=ON, MCLRE=OFF, CCP2MX=ON

/** C O N S T A N T E S ****************************************************/
//Vitesse defilement char editeur
const unsigned char CHG_SPEED = 250;

//Vitesses de defilement possibles
const unsigned int speed0 = 600;
const unsigned int speed1 = 250;
const unsigned int speed2 = 120;
const unsigned int speed3 = 80;
const unsigned int speed4 = 50;

//Etat de transition modes fonctions
const char MESSTEXT = 1;
const char REGLTEXT = 2;
const char MESSREGLH = 3;
const char REGLH = 4;
const char MESSREGLDATE = 5;
const char REGLDATE = 6;
const char MESSREGLAL = 7;
const char REGLAL = 8;

/** V A R I A B L E S ******************************************************/
#pragma idata

//Donnees du buffer USB
DATA_PACKET usbuff;

//nb d'octets a emettre sur le bus Usb
unsigned int send_oc;

//musique ecoutee en cours de debogage sur Chill et 1onAir.lounge
//Daft Punk - "Something about us"
//M83 - "You,Appearing"
//Snooze - "malandro" (album goingmobile)

//Buffer ecran et pointeur
unsigned char colon;				// ptr de colonne pour raf. video rapide

unsigned int scrptr;				// ctr (de 7 a 0) avant affichage nouveau char a ecrire
						// a droite de l'ecran dans buffer screen (si =0)

unsigned int screen[26];			// Image buffer ecran + 1 lettre d'avance

//Buffer message ascii (ou autre) a interpreter
//unsigned char msg[]; en rom
unsigned int msgptr;				// ptr prochaine lettre dans le message (en eeprom data)
unsigned char fontptr;				// ptr du char dans lib fonte
unsigned int speed,defiltime;		// vitesse en cours & ctr de defilement

unsigned char wide;					// largeur de fonte en cours (6,8) en colonne

unsigned char lasttmr;	// loops for timings...

unsigned int push_ctr;				//ctr push avant entre mode program encodeur 

unsigned char msg2[32];				// Buffer ascii utilise pour affichage des valeurs numerique
									// en direct (si msg2[0]<>ff). Chaine finie par 0xff.
									// La chaine2 est decalee a gauche a chaque fois.
unsigned char msg2ptr;				// Pointeur ecriture dans msg2[.]
unsigned int souligne;				// clignotement curseur souligne
unsigned char ordre;				// cde recue par USB
unsigned char inverse;
unsigned char ramoli;				// tempo select char

//Pour la date complete
unsigned int yyyy;
char mm, dd;
unsigned char hrs, mins, secs;

//Pour le reveil
unsigned char alhrs, almins;

//Semaphore = etape du fonctionnement/reglage en cours
unsigned char etat;

//Divers
unsigned char tt, s, n;
unsigned char charedit;			// le char en cours edition
FLAGBITS flags;					// flags divers (controlcode,inverse)

double xx, yy, moytemp;

#pragma udata usb7
//Tampon du texte recu par liaison USB (en memoire usb ram pour faire de la place!)
unsigned char txtbuff[200];

/** D E C L A R A T I O N S *************************************************/
#pragma code

/****************************************************************************
 * Initialization ports, timer1, Adcon
 *
 ****************************************************************************/
void UserInit(void) {
//!!! NE PAS RALENTIR AVANT L'ENUM USB !!!

 PORTA = 0xff;
 TRISA = 0b11100011;

 PORTB = 0xff;
 TRISB = 0b10000000;			// Sortie 7 lignes (out) + Ksetup (in)
 OpenPORTB(PORTB_CHANGE_INT_OFF & PORTB_PULLUPS_ON);

 PORTC = 0xff;
 TRISC = 0b00111011;

 OpenTimer0( T0_16BIT & T0_PS_1_8 & T0_SOURCE_INT & TIMER_INT_OFF);
 WriteTimer0(0x8000);

 //Timer1 quartz 32,768khz pour avoir Heure
 OpenTimer1( TIMER_INT_ON & T1_8BIT_RW &
	T1_SOURCE_EXT & T1_PS_1_1 & T1_OSC1EN_ON & T1_SYNC_EXT_OFF );
 WriteTimer1(0x8000);
 
 //Remet heure & date a zero si POR
 if (isPOR) {
    secs=0; hrs=0; mins=0;
    alhrs=12;  almins=0;
    dd = 1;
    mm = 5;
    yyyy = 2009;
 }

 //Configure AD0 pour temperature
 ADCON1 = 0x0E;
 ADCON2 = 0x22;		// Fosc/32 , 4Tad
 ADCON0 = 0x01;		// ADC on sur AN0
 moytemp = 0;

 //Enable IT timer clock
 PIE1bits.TMR1IE = 1;
 INTCONbits.PEIE = 1;
 INTCONbits.GIE = 1;

 //Init encoder
 push_ctr = 0;
 //Pas mode edition, aff normal
 etat = 0;
 souligne = 0;
 wide = 8;
 flags.controlcode = 0;		// pas char special en cours
 flags.inverse = 0;			// pas video inverse
 flags.alarmon = 0;			// pas de reveil
 flags.sonne = 0;			// sonne pas !

 //init ptr char to display, et efface ecran, vitesse par defaut
 msgptr = 0;
 scrptr = 0;

 //Efface l'ecran & buffer val numeriques
 Cls();

 //Vitesse defilement 2 par defaut
 speed = speed2;
 defiltime = speed;

 RAZ1=0; RAZ2=1;
}

/******************************************************************************
 * Function:        void ProcessIO(void)
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *****************************************************************************/
void ProcessIO(void) {   
unsigned char lsb, ordrelen;
unsigned int col;

 //Si Buzzer du reveil ?
 if (flags.sonne==1) {
   if ( (!K_setup)||(!K_plus)||(!K_moins) )
     flags.sonne = 0;
   else
     //Sonne une seconde / 2
     if (secs & 1) Beep();
 }

 //Envoie l'ecran sur les leds / Synchro sur Timer sans its.
 lsb = TMR0L;
 if (TMR0H != lasttmr) {
  lasttmr = TMR0H;

  //Pour clignotement curseur editeur
  souligne++;

  //Pour balayage Video colonne
  colon++;
  if (colon>9)
   { RAZ1=0; RAZ2=1; }
  else
   { RAZ1=1; RAZ2=0; }
  if (colon>18) colon=0;	// 18-1 ?
  CLOCK=0; CLOCK=1;
  col = screen[colon + 7];
  LATB = col;
  LATA = (col>>5)&(16+8+4);

  //---------- Affichage defilant normal uniquement ---------------
  if ( !etat ) {
   //Voit si une periode de temps pour Shift une Colonne
   defiltime--;
   if (!defiltime) {
     defiltime = speed;
     //Decale tout screen a gauche (droite en memoire)
     for (s=25; s>0 ; s--)
       screen[s]=screen[s - 1];
    
     if (!scrptr) {
       // Il y a un char a afficher !
       scrptr = wide - 1;

	   //Teste si une valeur num en cours d'affich dans msg2()
       if (msg2[0]) {
	     n = msg2[0];
	     //Decale tout a gauche
         for (s=0; s<32; s++) msg2[s]=msg2[s + 1];
         msg2[31] = 0;
       }
	   else {
         //Cherche la prochaine lettre dans message texte eeprom
         n = RdEEPROM(msgptr++);
         //Texte termin => retour a la ligne !
         if ( (!n)||(n==0xff) )  {
           msgptr=0;
           scrptr = 1;	// chargera prochain char a prochaine colonne
         }
       }

       //---Si Char de controle # en cours de traitement ----------
       // #H -> Affichage de l'heure du type HH:MM:SS
       // #D -> Affichage de la date Joursemaine dd mois annee en texte
       // #I -> Inverse video
       // #1,#2,#3,#4 -> Vitesse lente a rapide
       // #P -> Petits caracteres de 6 colonnes
       // #N -> Caracteres normaux 8 colonnes
       // #T -> Aff la temperature
       // #B -> Emission d'un beep

       if (flags.controlcode) {
        flags.controlcode = 0;
        switch (n) {
        //---------Changement de Vitesse dfilement--------
        case '0':{ speed = speed0;  break; }
        case '1':{ speed = speed1;  break; }
        case '2':{ speed = speed2;  break; }
        case '3':{ speed = speed3;  break; }
        case '4':{ speed = speed4;  break; }

		//--------------changements de Fontes--------------
        case 'P':{ wide = 6;  break; }
        case 'N':{ wide = 8;  break; }

        //------------- Bascule Video Inverse--------------
        case 'I':{
          flags.inverse ^= 1;
          break;
        }
        //---------------- Beep du Buzzer -----------------
        case 'B':{
          Beep();
          break;
        }
        //-------------Affichage complet Heure-------------
        case 'H':{
          msg2ptr = 0;
          InttoStr2(hrs,0);
          msg2[msg2ptr++] = ':';
          InttoStr2(mins,1);
	      msg2[msg2ptr++] = ':';
          InttoStr2(secs,1);
	      msg2[msg2ptr++] = 0;
          break;
        }
        //-----------Affichage 'Temperature'---------------
        case 'T': {
          ADCON0bits.GO = 1;
          while (ADCON0bits.GO) { lsb=ADRESH; }
          
          // Formule T en dixieme degres
          // T (0.1C) = -0,2339x2 + 7,6811x + 405,14
          yy = (double)(ADRES);
          yy /= 256;
          xx = (-0.2339*yy*yy);
          xx += (7.6811*yy) + 400;
          //Moyenne la temprature sur 3 mesures 
          if (moytemp!=0) xx=(xx + 2*moytemp)/3;
          msg2ptr = 0;
          //Si temp ngative !
          if (xx < 0) {
            msg2[msg2ptr++] = '-';
            xx = -xx; 
          }
          moytemp = xx;

          col = (int)(xx/10);
          InttoStr2( col , 0);
          msg2[msg2ptr++] = '.';
          col = col % 10 ;
          InttoStr2( col , 0);
          msg2[msg2ptr++] = 126;       // 
          msg2[msg2ptr++] = 'C';       // C
          msg2[msg2ptr] = ' ';         //
          break;
        }
        //Affichage date et jour complet
        case 'D': {
          msg2ptr = 0;
          n = weekday();
          s=0;
          while (jours[n][s])  msg2[msg2ptr++] = jours[n][s++];
	      msg2[msg2ptr++] = ' ';
          //Jour du mois
          InttoStr2(dd,0);
          msg2[msg2ptr++] = ' ';
          //Le mois en texte
          s=0;
          while (mois[mm-1][s])  msg2[msg2ptr++] = mois[mm-1][s++];
	      msg2[msg2ptr++] = ' ';
	      //Annee
          InttoStr4(yyyy,1);
          msg2[msg2ptr] = 0;
          break;
        }
        default : {
          msg[msgptr++] = '#';
        }
        }
       } // fin if(controlcode)
       else {
         //Sinon teste si nouveau code de controle
         if (n=='#')
           flags.controlcode = 1;
         else
           //Si non controle, Affiche le char a droite de l'ecran (gauche en memoire)
           Printat(n,0);
       }
     } // fin scrptr
     else
      scrptr--;
   }  // fin if (!defiltime...)
  }   // fin if (etat==0...)


  //----Affichage defilant que pendant Annonces nouvel etat (avant reglages...)
  //--------- ou pendant defilement normal --------------------------
  if ( (etat==MESSTEXT)||(etat==MESSREGLH)||(etat==MESSREGLAL)||(etat==MESSREGLDATE) ) {
   //Voit si une periode de temps pour Shift une Colonne
   defiltime--;
   if (!defiltime) {
     defiltime = speed;
     //Decale tout screen a gauche (droite en memoire)
     for (s=25; s>0 ; s--)
       screen[s]=screen[s - 1];
    
     if (!scrptr) {
       // Il y a un char a afficher !
       scrptr = wide - 1;

	   //Prend le next char de msg2[.] (annonce de mode)
       n = msg2[msgptr++];

       if (!n) msgptr=0; else Printat(n,4);
     } // fin scrptr
     else
      scrptr--;
   }  // fin if (!defiltime)
  }   // fin if (etat==MESS...)

  //=========================================================
  //   TEST SI ENTREE MODE PROGRAM K_SETUP(> 3 sec)
  //=========================================================
  if ( (!etat) && (push_ctr>9000) ) {
    //Affiche 'Reglage Texte'
    etat = MESSTEXT;
    //Efface ecran
    Cls();
    //Met le message 'Edition Texte' dans msg2[.]
    SetMessage(1); 
    //Mode editeur -> petite fonte
    wide = 6;
    //debut message
    msgptr=0;
    push_ctr = 0;				// raz ctr
    Antireb();
  }

  //=========================================================
  //       ANNONCE EN COURS - MODE REGLAGE TEXTE
  //=========================================================
  if ( (K_setup) && (etat==MESSTEXT) && (push_ctr>0) ) {
    //Lance le mode program (avec anti-rebond)
    Antireb();
    //Lance reglage texte
    etat = REGLTEXT;
    //Efface ecran
    Cls();
    //Prend char a editer -> debut message
    msgptr=0;
    charedit = RdEEPROM(msgptr);
    push_ctr = 0;				// raz ctr
  }

  //=======================================================
  //            Mode EDITEUR DU TEXTE
  //=======================================================
  if (etat == REGLTEXT) {
    //Test si bouton Plus & Moins & aff la valeur
    Selection (32 , 127);
    
    //Appui court Setup -> valid lettre ?
    if ( (K_setup)&&(push_ctr>0) ) {		// ! ?
      //Sauve la lettre en EEprom
      WrEEPROM(msgptr++, charedit);
      //Force effacement curseur
      souligne=0;
      Printat(charedit, 8);
      //Prend next char
      charedit = RdEEPROM(msgptr);
      if ( (charedit>127)||(charedit<32) ) charedit = ' ';
      //Decale ecran un char a gauche ( 6 colonnes )
      for (n=25 ; n>8 ; n--) { screen[n]=screen[n-6]; }
      //Anti-rebond touche Setup
      Antireb();
    }

    //Appui tres long sur Setup -> Fin Edition !
    if ( (!K_setup)&&(push_ctr>5000) ) {
      push_ctr=0;
      //Maintenant reglage heure (message d'annonce)
      etat = MESSREGLH;
      Cls();
      //Marque la fin du texte
      WrEEPROM(msgptr, 0);
      //Met le message 'reglage heure' dans msg2[.]
      SetMessage(2);
      //Prend prem char
      msgptr = 0;
      charedit = msg2[0];		//Premier char a editer
      //Anti-rebond touche Setup
      Antireb();
    }
  }  // fin if(editeur)

  //=========================================================
  //         ANNONCE 'REGLAGE HEURE' EN COURS
  //=========================================================
  if ( (K_setup) && (etat==MESSREGLH) && (push_ctr>2) ) {
    //Lance le mode program (avec anti-rebond)
    Antireb();
    //Lance Reglage Heure
    etat = REGLH;
    //Efface ecran
    Cls();
    //Met l'heure comme texte dans msg2[]
    msg2ptr = 0;
    InttoStr2(hrs, 1);		// aff les heures
    msg2[msg2ptr++] = ':';
    InttoStr2(mins, 1);		// aff minutes avec '0'
    //msgptr sera le pointeur de digit de l'heure
    msgptr = 0;
    charedit = msg2[0];		//Premier char a editer
    push_ctr = 0;				// raz ctr
  }

  //=======================================================
  //           Mode Reglage de l'HEURE
  //=======================================================
  if ( etat == REGLH ) {
    //Test si bouton Plus & Moins & aff la valeur
    //De 0 a 2 pour dizaines heures
    switch (msgptr) {
    case 0 : { Selection('0','2');  break; }
    //De 0 a 5 pour dizaines minutes
    case 3 : { Selection('0','5'); break; }
    //De 0 a 9 pour unites min
    case 4 : { Selection('0','9'); break; }
    //De 0 a 9 (ou de 0 a 3 si >20) pour unites heures
    case 1 : { if (msg2[0]==2)
      Selection('0' , '3'); else Selection('0' , '9'); }
    }

    //Appui court Setup -> valid digit ?
    if ( (K_setup)&&(push_ctr>0) ) {
      //Sauve le digit (vrai decimal)
      msg2[msgptr] = charedit - '0';

      //Affiche le char non souligne
      souligne=0;
      Printat(charedit,8);      //Decale ecran un char a gauche ( 6 colonnes )
      for (n=25 ; n>8 ; n--) { screen[n]=screen[n-6]; }

      //Chiffre suivant (passe le ':' et stop a 5)
      msgptr++;
      charedit = msg2[msgptr];

      //Saute le ':' (non reglable)
      if ( msg2[msgptr]==':' ) {
        Printat(':',8);
        //Decale ecran un char a gauche ( 6 colonnes )
        for (n=25 ; n>8 ; n--) { screen[n]=screen[n-6]; }
        msgptr++;
        charedit = msg2[msgptr];
        Printat(charedit,8);
      }
      //Sort apres le 5 caracteres -> Annonce Reglage Alarme
      if ( msgptr > 4) {
        //Etape suivante -> affich 'regl Date'
        etat = MESSREGLDATE;
        push_ctr=0;
        //Sauve la nouvelle heure
        hrs = (10*msg2[0]) + msg2[1];
        mins = (10*msg2[3]) + msg2[4];
        //Aff annonce 'Reglage Date'
        Cls();
        SetMessage(5);
        msgptr=0;
      }
      //Anti-rebond touche Setup
      Antireb();
    }
  }  // fin if(setclock)

  //=========================================================
  //         ANNONCE 'REGLAGE DATE' EN COURS
  //=========================================================
  if ( (K_setup) && (etat==MESSREGLDATE) && (push_ctr>1) ) {
    //Lance le mode program (avec anti-rebond)
    Antireb();
    //Lance Reglage Heure
    etat = REGLDATE;
    //Efface ecran
    Cls();
    //Met la date en chiffre comme texte dans msg2[]
    msg2ptr = 0;
    //Jour du mois
    InttoStr2(dd,1);
    msg2[msg2ptr++] = '/';
    //Mois
    InttoStr2(mm,1);
    msg2[msg2ptr++] = '/';
    //Annee
    InttoStr4(yyyy,1);
    msg2[msg2ptr] = 0;

    //msgptr sera le pointeur de digit de l'heure
    msgptr = 0;
    charedit = msg2[0];		//Premier char a editer
    push_ctr = 0;				// raz ctr
  }

  //=======================================================
  //           Mode Reglage de la DATE
  //=======================================================
  if ( etat == REGLDATE ) {
    //Test si bouton Plus & Moins & aff la valeur
    switch (msgptr) {
   //Dizaines jours
    case 0 : { Selection('0','3');  break; }
    //Unites jours
    case 1 : { if (msg2[0]==3) Selection('0','1'); else Selection('0','9');
      break; }
    //Dizaines mois
    case 3 : { Selection('0','1'); break; }
    //Unites mois
    case 4 : { Selection('0','9'); break; }
    //Milliers annees
    case 6 : { Selection('1','2'); break; }
    //Pour le reste de '0' a '9'
    default : Selection('0','9');
    }
   
    //Appui court Setup -> valid digit ?
    if ( (K_setup)&&(push_ctr>1) ) {
      //Sauve le digit (vrai decimal)
      msg2[msgptr] = charedit - '0';

      //Affiche le char non souligne
      souligne=0;
      Printat(charedit,8);      //Decale ecran un char a gauche ( 6 colonnes )
      for (n=25 ; n>8 ; n--) { screen[n]=screen[n-6]; }

      //Chiffre suivant (passe le ':' et stop a 5)
      msgptr++;
      charedit = msg2[msgptr];

      //Saute le '/' (non reglable)
      if ( msg2[msgptr]=='/' ) {
        Printat('/',8);
        //Decale ecran un char a gauche ( 6 colonnes )
        for (n=25 ; n>8 ; n--) { screen[n]=screen[n-6]; }
        msgptr++;
        charedit = msg2[msgptr];
        Printat(charedit,8);
      }

      //Sort au 10 caracteres -> Annonce Reglage Alarme
      if ( msgptr > 9) {
        //Etape suivante -> affich 'regl alarme'
        etat = MESSREGLAL;
        push_ctr=0;
        //Sauve la nouvelle date
        dd = (10*msg2[0]) + msg2[1];
        mm = (10*msg2[3]) + msg2[4];
        yyyy = (1000*msg2[6]) + (100*msg2[7]) + (10*msg2[8]) + msg2[9];
        //Aff annonce 'Reglage alarme'
        SetMessage(3);
        msgptr=0;
      }
      //Anti-rebond touche Setup
      Antireb();
    }
  }  // fin if(setclock)

  //=========================================================
  //          ANNONCE 'REGLAGE ALARME' EN COURS
  //=========================================================
  if (etat==MESSREGLAL) {
   //Aff annonce 'Alarme active' (ou non) suivant etat
   if (flags.alarmon) SetMessage(3); else SetMessage(4);
   
   //Inversion etat ON/OFF alarme
   if  ( (!K_moins)||(!K_plus) ) {
     Cls();
     msgptr = 0;
     flags.alarmon = (flags.alarmon)?0:1;
     //Anti-rebond (333mS)
     Delay10KTCYx(200);
   }

   //Validation etat on/off alarme ?
   if  ( (K_setup)&&(push_ctr>1) ) {
     //Lance le mode program (avec anti-rebond)
     Antireb();
     //Efface ecran
     Cls();
     //Si pas alarme par defaut -> fin reglages
     etat = 0;
     if (flags.alarmon) {
       //Debut reglage alarme
       etat = REGLAL;
       //Met l'heure comme texte dans msg2[]
       msg2ptr = 0;
       InttoStr2(alhrs, 1);		// aff les heures
       msg2[msg2ptr++] = ':';
       InttoStr2(almins, 1);		// aff minutes avec '0'
       //prend premier char digit de l'alarme
       charedit = msg2[0];		//Premier char a editer
     }
     msgptr = 0;
     push_ctr = 0;
   }
  }

  //=======================================================
  //              Mode Reglage ALARME
  //=======================================================
  if ( etat == REGLAL ) {
    //Test si bouton Plus & Moins & aff la valeur
    //De 0 a 2 pour dizaines heures
    switch (msgptr) {
    case 0 : { Selection('0','2');  break; }
    //De 0 a 5 pour dizaines minutes
    case 3 : { Selection('0','5'); break; }
    //De 0 a 9 pour unites min
    case 4 : { Selection('0','9'); break; }
    //De 0 a 9 (ou de 0 a 3 si >20) pour unites heures
    case 1 : { if (msg2[0]==2)
      Selection('0' , '3'); else Selection('0' , '9'); }
    }
   
    //Appui court Setup -> valid digit ?
    if ( (K_setup) && (push_ctr>2) ) {
      //Sauve le digit (vrai decimal)
      msg2[msgptr] = charedit - '0';

      //Affiche le char non souligne
      souligne=0;
      Printat(charedit,8);      //Decale ecran un char a gauche ( 6 colonnes )
      for (n=25 ; n>8 ; n--) { screen[n]=screen[n-6]; }

      //Chiffre suivant (passe le ':' et stop a 5)
      msgptr++;
      charedit = msg2[msgptr];

      //Saute le ':' (non reglable)
      if ( msg2[msgptr]==':' ) {
        Printat(':',8);
        //Decale ecran un char a gauche ( 6 colonnes )
        for (n=25 ; n>8 ; n--) { screen[n]=screen[n-6]; }
        msgptr++;
        charedit = msg2[msgptr];
        Printat(charedit,8);
      }

      //Sort apres le 5 caracteres -> Fin Reglage alarme
      if ( msgptr > 4) {
        //Termine apres plus de reglages !
        etat = 0;
        push_ctr=0;
        //Par defaur fonte big
        wide=8;
        //Sauve la nouvelle heure reveil
        alhrs = (10*msg2[0]) + msg2[1];
        almins = (10*msg2[3]) + msg2[4];
        //Repart debut
        Cls();
        msgptr=0;
      }
      //Anti-rebond touche Setup
      Antireb();
    }
  }  // fin if(setclock)

  //Comptage Temps de Bouton Setup appuye
  if (!K_setup) push_ctr++; else push_ctr=0;
 } // fin  if (TMR...)

//=========================================================
//         TEST RECEIVE USB COMMAND ?
//=========================================================
 if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
 ordrelen=(USBGenRead((byte*) & usbuff , USBGEN_EP_SIZE));

 //Check the if a good prefix command received
 if (ordrelen!=0) ordre = usbuff._byte[0];

//=========================================================
//       Lecture 64-2 octets de message dataEEPROM
// cmd = 0x85 - debut ==> repond un paquet de 64 octets 
//=========================================================
 if (ordre==0x85) {
   tt=usbuff._byte[1];
   //Pour accuse reception
   usbuff._byte[0]=0xAA;
   //Envoi 62 octets donnees
   for (n=2; n<USBGEN_EP_SIZE ; n++) usbuff._byte[n]=RdEEPROM(tt++);
   send_oc = USBGEN_EP_SIZE;
   ordre = 0;
   msgptr = 0;
 }

//=========================================================
//       Ordre d'criture Texte en dataEEPROM
// Cmd = 0x58 - addr dest - 64 octets char maxi ...
//=========================================================
 if (ordre==0x58) {
   msgptr=0;
   // tt = adr destination du paquet en eeprom
   tt=usbuff._byte[1];
   //recopie d'abord tout les morceaux dans txtbuff[255]
   for (n=2; n<USBGEN_EP_SIZE ; n++) txtbuff[tt++]=usbuff._byte[n];
   
   //Voit si tout est transmi -> recopie dans l'EEPROM
   if (usbuff._byte[USBGEN_EP_SIZE-1]==0) {
     //Stoppe toutes les Its !
     INTCONbits.GIE=0;
     for (n=0; n<200 ; n++) {
       WrEEPROM(n , txtbuff[n]);
       if (!txtbuff[n]) break;
     }
     //Remet toutes les Its !
     INTCONbits.GIE=1;
   }
   ordre=0;						// ordre trait
   flags.controlcode = 0;		// pas char special en cours
   flags.inverse = 0;			// pas video inverse
 }

//=========================================================
//         Ordre de lecture de l'heure+date+alarme
// cmd = 0x43 - debut ==> repond un paquet de 64 octets
// 0xAA.hh.mm.ss dd.mm.yyyy(16bits).alhr.almin.alonoff 
//=========================================================
 if (ordre==0x43) {
   //Pour accuse reception
   usbuff._byte[0]=0xAA;
 
   //Met l'heure
   usbuff._byte[1]= hrs;
   usbuff._byte[2]= mins;
   usbuff._byte[3]= secs;

   //Met la date
   usbuff._byte[4]= dd;
   usbuff._byte[5]= mm;
   usbuff._byte[6]= (yyyy / 256);
   usbuff._byte[7]= (yyyy % 256);

   //Met l'heure alarme
   usbuff._byte[8]= alhrs;
   usbuff._byte[9]= almins;
   //Flag si alarme on/off
   usbuff._byte[10]= (flags.alarmon==1)?1:0;

   //Renvoie resultat
   send_oc = USBGEN_EP_SIZE;
   ordre = 0;
   msgptr = 0;
 }

//=========================================================
//         Ordre de Programmation de l'Heure+Date
// cmd = 0x34.hh.mm.ss dd.mm.yyyy(16bits) 
//=========================================================
 if (ordre==0x34) {
   hrs = usbuff._byte[1];
   mins = usbuff._byte[2];
   secs = usbuff._byte[3];

   dd = usbuff._byte[4]= dd;
   mm = usbuff._byte[5]= mm;
   yyyy = (256 * usbuff._byte[6]) + usbuff._byte[7];

   //Reprend l'heure alarme
   alhrs = usbuff._byte[8];
   almins = usbuff._byte[9];
   //Flag si alarme on/off
   flags.alarmon = (usbuff._byte[10])?1:0;

   //Pour accuse reception
   usbuff._byte[0] = 0xAA;
   send_oc = USBGEN_EP_SIZE;
   ordre = 0;
   msgptr = 0;
 }

//=========================================================
//         VOIT SI RENVOI DE DONNEES EN RETOUR
//=========================================================
 if (send_oc >= USBGEN_EP_SIZE) {     
   if(!mUSBGenTxIsBusy()) {
     USBGenWrite(&usbuff, USBGEN_EP_SIZE);
     send_oc -= USBGEN_EP_SIZE;
   }
 }
}//end ProcessIO

/************************************************************/
//   Selection d'une valeur evec K_PLUS et K_MOINS
// * affiche la valeur
// * entre et sort dans Charedit de (min) a (max) (inclus)
// * pas de gestion de sortie juste + et -
/************************************************************/
unsigned char Selection(unsigned char min, unsigned char max) {
 //Test si bouton Plus
 if ( (!ramoli)&&(!K_plus) ) {
   charedit++;
   if ( (charedit>max)||(charedit<min) ) charedit=min;
   //Tempo Anti-Rebond clavier
   Delay10KTCYx(40);
   ramoli = CHG_SPEED;
 }
     
 //Test si bouton Moins
 if ( (!ramoli)&&(!K_moins) ) {
   charedit--;
   if ( (charedit<min)||(charedit>max) ) charedit=max;
   //Tempo Anti-Rebond clavier
   Delay10KTCYx(40);
   ramoli = CHG_SPEED;
 }
 Printat(charedit, 8);
 if (ramoli) ramoli--;
}

/************************************************************/
// Affiche le char ascii (asc) a la colonne (pos) dans buffer
//
// - si modes reglages affiche le soulign clignotant.
/************************************************************/
void Printat(unsigned char asc, unsigned char pos) {
unsigned char fptr, nn;
unsigned int mask;

 fptr = 0;
 mask = 0;

 //Dessin du Curseur clignotant editeur(s)
 if ( (etat==REGLTEXT)||(etat==REGLH)||(etat==REGLAL)||(etat==REGLDATE) ) {
   souligne++;
   if (souligne & 256) mask=0x0200;
 }
 //Dessin Video inversee
 if (flags.inverse) mask=0x03ff;

 //=========== SI MODE PETITE FONTE ============
 if (wide==6) {
  //Passe en Largeur en 6 de large
   while (1) {
     //Si caractere ascii inconnu -> met SPC
     if (car6[fptr].chr_asc==0) {
       fptr = 0;
       break;
     }
     if (car6[fptr].chr_asc == asc) break; else fptr++;
   }
   //Recopie la lettre de font.h a screen[pos]
   for (nn=0; nn<wide; nn++)
     screen[pos+nn] = mask^car6[fptr].chr_def[wide-1-nn];
 }
 else {
 //=========== Sinon 8 colonnes de large ======
   while (1) {
     //Si caractere ascii inconnu -> met SPC
     if (car8[fptr].chr_asc==0) {
      fptr = 0;
      break;
     }
     if (car8[fptr].chr_asc == asc) break; else fptr++;
   }
   //Recopie la lettre de font.h a screen[pos]
   for (nn=0; nn<wide; nn++)
     screen[pos+nn] = mask^car8[fptr].chr_def[wide-1-nn];
 }
}

/********************************************************/
// Diverses fonctions
/********************************************************/
void Cls() {
 //Efface l'ecran
 for (s=0; s<26; s++) screen[s]=0xffff;
 //Vide buffer val numerique
 for (s=0; s<32; s++) msg2[s]=0;
}

// Antirebond tempo bloquant sur K_setup
void Antireb() {
 while (!K_setup) { Delay10KTCYx(10); }
 Delay10KTCYx(100);
}

void Hex(unsigned int val) {
//Retourne val en hexa dans msg2[...] 
unsigned char d, u;
 d = val >> 8;
 u = d /16;
 u += (u>9)?55:48;
 msg2[msg2ptr++] =  u;
 u = d % 16;
 u += (u>9)?55:48;
 msg2[msg2ptr++] =  u;

 d = val % 256;
 u = d/ 16;
 u += (u>9)?55:48;
 msg2[msg2ptr++] =  u;
 u = d % 16;
 u += (u>9)?55:48;
 msg2[msg2ptr++] =  u;
}

//Fait un beep sur RA1 (3kHz pendant 0,5 sec) 
void Beep() {
 TRISA = 0b11100001;
 for (n=0; n<50 ; n++) {
  for (tt=0 ; tt<30 ; tt++) {
    BUZZ=0;
    Delay10TCYx(100);
    BUZZ=1;
    Delay10TCYx(100);
  }
 }
 TRISA = 0b11100011;
}

/************************************************************/
// Affiche un message d'annonce dans msg2[.]
//
/************************************************************/
void SetMessage(unsigned char no) {
const rom char msg[6][28] = {
 {"Electronique Pratique"},{"EDITION TEXTE  "},
 {"REGLAGE HEURE  "},{"ALARME ACTIVE "},{"PAS D'ALARME "},
 {"REGLAGE DATE   "} };

 n=0;

 while (msg[no][n]) { 
   msg2[n]=msg[no][n];
   n++;
 }
 msg2[n] = 0; 
}

/************************************************************/
// Affiche le int val en string dans msg2[] sur 2 char a la
// position de msg2ptr.
// * msg2ptr est increment en sortie
// * Si zero<>0 affiche le zero au debut
/************************************************************/
void InttoStr4(unsigned int val,unsigned char zero) {
unsigned int d, u;

 //Fait les Milles
 d = (val/1000);
 val = (val % 1000);
 //Voit si zero des Milles
 if ( (d)||(zero) ) {
   msg2[msg2ptr++] = '0' + d;
   zero = 1;
 }

 //Fait les Centaines
 d = val/100;
 val = val % 100;
 //Voit si zero des cents
 if ( (d)||(zero) ) {
   msg2[msg2ptr++] = '0' + d;
   zero = 1;
 }

 //Fait les dizaines
 u = val % 10;
 d = val/10;
 //Voit si zero des dizaines
 if ( (d)||(zero) ) msg2[msg2ptr++] = '0' + d;

 //Met units
 msg2[msg2ptr++] = '0' + u;
 //Marque la fin
 msg2[msg2ptr] = 0;
}

//Pour 2 chiffres 
void InttoStr2(unsigned int val,unsigned char zero) {
unsigned char d, u;

 //Fait les dizaines
 u = val % 10;
 d = val/10;
 //Voit si zero des dizaines
 if ( (d)||(zero) ) msg2[msg2ptr++] = '0' + d;

 //Met units
 msg2[msg2ptr++] = '0' + u;
 //Marque la fin
 msg2[msg2ptr] = 0;
}
/***********************************************************/
