/**************************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 **************************************************************************
 * FileName:        user.c
 * Processor:       PIC18
 *************************************************************************/

/** I N C L U D E S *******************************************************/
#include "p18f2550.h"
#include "typedefs.h"
#include "usb.h"
#include "user.h"
#include "interrupt.h"
#include "delays.h"

#pragma code page

//**********************************************************************
// Utilis pour connaitre le nb de jours dans le mois
//**********************************************************************
unsigned char jourmois(void){
const char jm[13]={0,31,28,31,30,31,30,31,31,30,31,30,31};

 if(((yyyy%4==0) && (yyyy%100!=0)) || (yyyy%400==0)) 
   if (mm==2)
      return (29);
   else
      return (jm[mm]);
 else
   return (jm[mm]);
}

/*(d+=m<3?y--:y-2, 23*m/9 + d + 4 + y/4 -y/100 + y/400)  7 

Mike Keith dclare lui-mme qu'il s'agit l de la " plus simple formule possible 
pour le calcul du jour de la semaine, " la meilleure qui puisse tre construite ", ajoute-t-il.
Dans sa forme oprationnelle, son algorithme s'crit : 
Jour de semaine D = { [(23m)/9] + d + 4 + y + [z/4] - [z/100] + [z/400] - 2 (si m >= 3) } mod 7 
 
D = Jour de semaine   (D=0  6; 0=Dimanche, 1=Lundi,...,6=Samedi) 
[x] signifie le nombre entier rsultant de la division, le reste tant ignor 
m = Mois   (m = 1  12; 1=Janvier,...,12=Dcembre)  m>=3 signifie m suprieur ou gal  3 
d = Jour   (d = 1  31) 
y = anne 
z = y - 1   si m<3 
z = y       si m>=3 
Modulo 7 ou mod 7 = reste de la division par 7  */

char weekday(void) {
int days;
int z;

 if (mm<3) z = yyyy-1; else z = yyyy;
 days = (23*mm)/9;
 days += dd + 4 + yyyy + (z/4)-(z/100)+(z/400);
 if (mm>=3) days -= 2;
 days %= 7;

 return (days);
}

/************************************************************************
 * Procedure de service d'interruption
 ***********************************************************************/
#pragma code
#pragma interrupt isr
void isr(void) {
 
 //==================================================================
 // Interruption du Timer 1
 //==================================================================
 if (PIR1bits.TMR1IF) {
  //Recharge timerH pour 1 sec

  _asm
   bsf	TMR1H,7,0
  _endasm

  //Augmente l'heure & la date
  secs++;
  if (secs==60) {
	secs=0;
    mins++;
	if (mins==60) {
	  mins=0;
      hrs++;
	  if (hrs==24) {
		hrs=0;
        dd++;
		if (dd>jourmois()) {
		  dd=1;
          mm++;
		  if (mm>12) {
			mm=1;
			yyyy++;
		  }
		}
      }
	}
  }

  //Teste si heure alarme reveil
  if (flags.alarmon==1) {
    if ((alhrs==hrs)&&(almins==mins)&&(secs==0))
       //Bascule la Sonnerie a active !
       flags.sonne = 1;
    if ((alhrs==hrs)&&(almins==mins)&&(secs>30))
       //Arrete apres 30 secondes d'office !
       flags.sonne = 0;
  }

  //Reset l'it du timer 1
  PIR1bits.TMR1IF=0;
 }
}

#pragma interrupt isr
void lowisr(void) {
}
#pragma
//===============================================================
